import numpy as np
import matplotlib.pyplot as plt
import glob
import sys

def find_consecutive_ones_positions(arr):
    positions = []
    start = None

    for i, num in enumerate(arr):
        if num == 1:
            if start is None:
                start = i
        elif start is not None:
            positions.append((start, i - 1))
            start = None

    # If the sequence of 1s continues till the end of the array
    if start is not None:
        positions.append((start, len(arr) - 1))

    return positions


if len(sys.argv) >1:
    spike_name_end = sys.argv[1]
else:
	spike_name_end = 'spikes'

fname_list = glob.glob('*{}.csv'.format(spike_name_end))
# fname_list = ['maturation_N_195_L_1.00_Ipatt_00_0_spikes.csv']


if len(sys.argv) >2:
    burst_counts = int(sys.argv[2])
    burst_time_bin = float(sys.argv[3])
else:
    burst_counts = 5
    burst_time_bin = 0.1
burst_frequency = burst_counts/burst_time_bin


if_print_figures = True

for fname in fname_list:
    filename_save = fname.split('.csv')[0]

    all_spikes = np.loadtxt(fname, delimiter=',')
    spike_id = all_spikes[:,0].astype(int)


    all_bursts = []
    for i in np.unique(spike_id):
        which_neuron = i
        tmp_id = (spike_id == which_neuron)

        tmp_spike_time = all_spikes[tmp_id,1]


        if_a_burst = np.zeros(tmp_spike_time.shape, dtype=int)
        for i in range(if_a_burst.shape[0]):
            current_time = tmp_spike_time[i]
            end_time = current_time + burst_time_bin
            if ((tmp_spike_time >= current_time) * (tmp_spike_time <= end_time)).sum() >= burst_counts:
                if_a_burst[i] = 1


        burst_events = find_consecutive_ones_positions(if_a_burst)
        # print("Positions of consecutive 1s:", burst_events)

        burst_data = np.zeros( (len(burst_events), 3) )
        for i in range(len(burst_events)):
            burst_data[i, 0] = which_neuron
            burst_data[i, 1] = tmp_spike_time[burst_events[i][0]]
            burst_data[i, 2] = tmp_spike_time[burst_events[i][1]]

        if len(all_bursts) == 0:
            all_bursts = np.copy(burst_data)
        else:
            all_bursts =  np.concatenate( (all_bursts,burst_data), axis=0 )


    sort_id = np.argsort(all_bursts[:,1])
    all_bursts = all_bursts[sort_id,:]

    header = 'ROI, Burst starts, Burst ends'
    np.savetxt(filename_save+'burst.csv', all_bursts, header=header, delimiter=',')


    all_bursts = all_bursts[:,:2]
    header = 'ROI, Burst starts'
    np.savetxt(filename_save+'burst_start_time.csv', all_bursts, header=header, delimiter=',')

    if if_print_figures:
        X = []
        Y = []
        for n in np.unique(all_bursts[:,0]):
            tmp_id = (all_bursts[:,0] == n)
            tmp_bursts = np.copy(all_bursts[tmp_id])
            tmp_bursts = np.c_[tmp_bursts, np.array([None for _ in tmp_bursts])]
            Y_tmp = tmp_bursts[:,1:].flatten()
            X_tmp = n*np.ones(Y_tmp.shape)
            Y.append(list(Y_tmp))
            Y.append([None])
            X.append(list(X_tmp))
            X.append([None])


        import itertools
        X = list(itertools.chain.from_iterable(X))
        Y = list(itertools.chain.from_iterable(Y))

        plt.close()
        plt.plot(  Y, X, c='b', linewidth=1, marker='|', markersize=1 )
        plt.savefig(filename_save+'burst.pdf')





        import plotly.graph_objs as go

        # Assuming X and Y are lists of coordinates
        # X contains the x-coordinates and Y contains the y-coordinates

        data = []

        trace = go.Scatter(
            x=Y,
            y=X,
            mode='lines+markers',
            marker=dict(
                color='blue',
                symbol='line-ns-open',
                size=6
            ),
            # line=dict(
            #     color='blue',
            #     width=1
            # )
            line=dict(
                color='rgba(0, 0, 255, 0.5)',  # Blue color with 50% transparency
                width=5  # Increase the width of the line
            ),
            name = 'burst'
        )
        data.append(trace)


        trace = go.Scatter(
            x=all_spikes[:,1],
            y=all_spikes[:,0],
            mode='markers',
            marker=dict(
                color='black',
                symbol='line-ns-open',
                size=3
            ),
            name='spikes',  # Add a name for the trace
            visible='legendonly'  # Set the visibility to legendonly by default
        )
        data.append(trace)


        layout = go.Layout(
            title='Burst Plot',
            xaxis=dict(title='time (s)'),
            yaxis=dict(title='ROI')
        )

        fig = go.Figure(data=data, layout=layout)

        # Save the plot as an HTML file
        fig.write_html(filename_save+'burst_plot.html', auto_open=False)
